import {Component, ViewChild} from '@angular/core'
import {currentPermissionsFieldComponent} from '../../../shared/components/permissions-field/permissions-field.component'
import {EmployeeService} from '../../services/employee.service'
import {GroupService} from '../../../groups/services/group.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Employee} from '../../../shared/model/employee'
import {AccountService} from '../../../accounts/services/account.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ErrorService} from '../../../shared/service/error.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {map} from 'rxjs/operators'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  @ViewChild('permissionsTemplateRef', {static: true}) permissionsTemplateRef

  employee: Employee | null
  formFields: FormField[] = [{
    name: 'id',
    label: 'Numéro d\'employé',
    icon: 'pound',
    type: 'number',
    readonly: true
  }, {
    name: 'firstName',
    label: 'Prénom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un prénom est requis'}
    ]
  }, {
    name: 'lastName',
    label: 'Nom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un nom est requis'}
    ]
  }, {
    name: 'groupId',
    label: 'Groupe',
    icon: 'account-multiple',
    type: 'select',
    valueFn: employee => employee.group ? employee.group.id : -1,
    options$: this.groupService.allWithDefault.pipe(map(groups => groups.map(g => {
      return {value: g.id, label: g.name}
    })))
  }, {
    name: 'permissions',
    label: 'Permissions',
    type: 'permissions'
  }]

  constructor(
    private accountService: AccountService,
    private employeeService: EmployeeService,
    private groupService: GroupService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.employeeService,
      id,
      employee => this.employee = employee,
      '/employee/list'
    )

    this.formFields[this.formFields.length - 1].template = this.permissionsTemplateRef
  }

  submit(values) {
    const permissionsField = currentPermissionsFieldComponent
    if (permissionsField.valid()) {
      this.subscribe(
        this.employeeService.update(
          parseInt(values.id),
          values.firstName,
          values.lastName,
          permissionsField.allEnabledPermissions
        ),
        () => {
          // TODO de-comment when backend will be ready
          // const group = values.groupId
          // if (group >= 0) {
          //   this.subscribeAndNavigate(
          //     this.groupService.addEmployeeToGroup(group, this.employee),
          //     '/employee/list'
          //   )
          // } else {
          //   if (this.employee.group) {
          //     this.subscribeAndNavigate(
          //       this.groupService.removeEmployeeFromGroup(this.employee),
          //       '/employee/list'
          //     )
          //   } else {
              this.urlUtils.navigateTo('/employee/list')
            // }
          // }
        }
      )
    }
  }

  delete() {
    this.subscribeAndNavigate(
      this.employeeService.delete(this.employee.id),
      '/employee/list'
    )
  }
}
